/*
 * SEE shell
 *
 * This shell provides a 'print' function object which takes
 * one string argument and prints it followed by a newline.
 * It evaluates the javascript inside each of its file (-f) options.
 * If no files are given, it prompts for javascript code interactively.
 */

#include <stdio.h>
#include <see/see.h>
#include "shell.h"

static void print_fn(struct SEE_interpreter *,
        struct SEE_object *, struct SEE_object *, int,
	struct SEE_value **, struct SEE_value *);

static void
print_fn(interp, self, thisobj, argc, argv, res)
        struct SEE_interpreter *interp;
        struct SEE_object *self, *thisobj;
        int argc;
        struct SEE_value **argv, *res;
{
        struct SEE_value v;

        if (argc) {
                SEE_ToString(interp, argv[0], &v);
                SEE_string_fputs(v.u.string, stdout);
		printf("\n");
		fflush(stdout);
        }
        SEE_SET_UNDEFINED(res);
}

void
shell_add_globals(interp)
	struct SEE_interpreter *interp;
{
	struct SEE_object *obj;
	struct SEE_value v;
	struct SEE_string *name;

	name = SEE_string_sprintf(interp, "print");
	obj = SEE_cfunction_make(interp, print_fn, name, 1);
	SEE_SET_OBJECT(&v, obj);
	SEE_OBJECT_PUT(interp, interp->Global, name, &v, 0);

	name = SEE_string_sprintf(interp, "version");
	SEE_SET_UNDEFINED(&v);
	SEE_OBJECT_PUT(interp, interp->Global, name, &v, 0);
}

static void
document_write(interp, self, thisobj, argc, argv, res)
        struct SEE_interpreter *interp;
        struct SEE_object *self, *thisobj;
        int argc;
        struct SEE_value **argv, *res;
{
        struct SEE_value v;

        if (argc) {
                SEE_ToString(interp, argv[0], &v);
                SEE_string_fputs(v.u.string, stdout);
		fflush(stdout);
        }
        SEE_SET_UNDEFINED(res);
}

void
shell_add_document(interp)
	struct SEE_interpreter *interp;
{
	struct SEE_object *obj, *document, *navigator;
	struct SEE_value v;
	struct SEE_string *name;

	name = SEE_string_sprintf(interp, "document");
	document = SEE_Object_new(interp);
	SEE_SET_OBJECT(&v, document);
	SEE_OBJECT_PUT(interp, interp->Global, name, &v, 0);

	name = SEE_string_sprintf(interp, "write");
	obj = SEE_cfunction_make(interp, document_write, name, 1);
	SEE_SET_OBJECT(&v, obj);
	SEE_OBJECT_PUT(interp, document, name, &v, 0);

	name = SEE_string_sprintf(interp, "navigator");
	navigator = SEE_Object_new(interp);
	SEE_SET_OBJECT(&v, navigator);
	SEE_OBJECT_PUT(interp, interp->Global, name, &v, 0);

	name = SEE_string_sprintf(interp, "userAgent");
	SEE_SET_STRING(&v, SEE_string_sprintf(interp, "SEE-shell"));
	SEE_OBJECT_PUT(interp, navigator, name, &v, 0);

	name = SEE_string_sprintf(interp, "window");
	SEE_SET_OBJECT(&v, interp->Global);
	SEE_OBJECT_PUT(interp, interp->Global, name, &v, 0);

}
